package net.lenharts.gwt.sampleiphoneapp.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;
import com.google.gwt.http.client.URL;
import com.google.gwt.user.client.Window;

import net.lenharts.gwt.sampleiphoneapp.client.model.GroupPage;
import net.lenharts.gwt.sampleiphoneapp.client.model.IPage;
import net.lenharts.gwt.sampleiphoneapp.client.model.InfiniteDepthPage;
import net.lenharts.gwt.sampleiphoneapp.client.model.MainPage;
import net.lenharts.gwt.sampleiphoneapp.client.model.TournamentPage;
import net.lenharts.gwt.sampleiphoneapp.client.util.ObservableStack;

public class Controller {

	private static boolean mIgnoreEvents = false;

	private static ObservableStack<IPage> mPageStack;

	private Controller() {

	}

	public static void init(ObservableStack<IPage> pageStack) {

		mPageStack = pageStack;

		downloadXml(GWT.getModuleBaseURL() + URL.encode("../data/data.xml"), new XmlHttpResponse() {
			@Override
			public void onSuccessfulReceive(String xmlText) {
				MainPage p = MainPage.convert(xmlText);
				mPageStack.push(p);
			}

			@Override
			public void onUnsuccessfulReceive(String error) {
				Window.alert(error);
			}
		});

	}

	public static void navigateBack() {
		mPageStack.pop();
	}

	public static void navigateToTournament(String relativeUrl, IPage currentPage) {
		if (!mIgnoreEvents) {
			mIgnoreEvents = true;

			downloadXml(GWT.getModuleBaseURL() + URL.encode("../data/" + relativeUrl),
					new XmlHttpResponse() {
						@Override
						public void onSuccessfulReceive(String xmlText) {
							TournamentPage p = TournamentPage.convert(xmlText);
							mPageStack.push(p);
							mIgnoreEvents = false;
						}

						@Override
						public void onUnsuccessfulReceive(String error) {
							Window.alert(error);

							mIgnoreEvents = false;

						}
					});
		}
	}

	public static void navigateToGroup(String relativeUrl, IPage currentPage) {
		if (!mIgnoreEvents) {
			final String currentTournamentName = currentPage.getTitle();

			mIgnoreEvents = true;
			downloadXml(GWT.getModuleBaseURL() + URL.encode("../data/" + relativeUrl),
					new XmlHttpResponse() {
						@Override
						public void onSuccessfulReceive(String xmlText) {
							GroupPage p = GroupPage.convert(xmlText,
									currentTournamentName);
							mPageStack.push(p);
							mIgnoreEvents = false;
						}

						@Override
						public void onUnsuccessfulReceive(String error) {
							Window.alert(error);

							mIgnoreEvents = false;

						}
					});
		}
	}

	public static void navigateToFirstInfiniteListItem() {
		if (!mIgnoreEvents) {
			InfiniteDepthPage p = new InfiniteDepthPage(mPageStack.peek()
					.getTitle());
			mPageStack.push(p);
		}
	}

	public static void navigateToNextInfiniteListItem(
			InfiniteDepthPage currentPage) {
		if (!mIgnoreEvents) {
			String currentTitle = currentPage.getTitle();
			int currentTitleNumber = currentPage.getMyTitleNumber();

			InfiniteDepthPage p = new InfiniteDepthPage(currentTitle,
					currentTitleNumber);
			mPageStack.push(p);

		}
	}

	private static void downloadXml(String url, XmlHttpResponse responseCallback) {

		final XmlHttpResponse finalResponseCallback = responseCallback;
		try {
			RequestBuilder req = new RequestBuilder(RequestBuilder.GET, url);
			req.sendRequest(null, new RequestCallback() {

				@Override
				public void onError(Request request, Throwable exception) {
					finalResponseCallback
							.onUnsuccessfulReceive("Unable to contact server");
				}

				@Override
				public void onResponseReceived(Request request,
						Response response) {
					if (400 > response.getStatusCode()
							&& 0 != response.getStatusCode()) {
						finalResponseCallback.onSuccessfulReceive(response
								.getText());
					} else {
						switch (response.getStatusCode()) {
						case 0:
							finalResponseCallback
									.onUnsuccessfulReceive("Unable to contact server");
							break;
						case 404:
							finalResponseCallback
									.onUnsuccessfulReceive("Data file not found");
							break;
						default:
							finalResponseCallback
									.onUnsuccessfulReceive("Error loading file: "
											+ response.getStatusCode());

						}
					}
				}

			});
		} catch (RequestException e) {
			responseCallback.onUnsuccessfulReceive(e.getMessage());
		}

	}

	private interface XmlHttpResponse {
		void onSuccessfulReceive(String xmlText);

		void onUnsuccessfulReceive(String error);
	}

}
